	SUBROUTINE READHUB
*******************************************************
*  Reads HUBFILE which contains Slater integrals      *
*  necessary for constructing U-matrix U(m' k'; m k)  *
*  in the LDA+U method.                               *
*  Also reads occupation numbers for correlated       *
*  states if avaiable.                                *
*  To make constrained LDA, specify corrections to    *
*  the LDA potential, which will be read below.       *
*******************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        INCLUDE 'PARAM.DAT'
        INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX)
     &   ,NKAP(NPANMAX),IATS(NPAT,NPANMAX),ISTS(NSORTMAX,NPANMAX)
     &   ,IS(NPAT),NHSORT(NSORTMAX),MCRL(NPAT),LCRL(0:LMAXB,NPAT)
        DIMENSION SMT(NSORTMAX),FHUB(0:LMAXB)
     &   ,FSLG(0:2*LMAXB,0:LMAXB,0:LMAXB,NPAT),OCRHUB(-LMAXB:LMAXB)
     &   ,FSLU(0:2*LMAXB,0:LMAXB,0:LMAXB,NPAT),OCRLDA(-LMAXB:LMAXB)
        COMPLEX*16 TS(7,-3:3,0:3),OCPHUB(LIMLMB2),OCPLDA(LIMLMB2)
     &   ,DHUB(LIMLMB2,NPAT,NSPINMAX),DLDA(LIMLMB2,NPAT,NSPINMAX)
     &   ,VHUB(LIMLMB2,NPAT,NSPINMAX),VCNS(LIMLMB2,NPAT,NSPINMAX)
        LOGICAL LSPAN(0:LMAXT+1,NSORTMAX,NKAPMAX,NPANMAX)
     &   ,LTPAN(0:LMAXT+1,NPAT,NKAPMAX,NPANMAX),EXIST
        CHARACTER*60 HUBFILE
        CHARACTER TXTEL(NSORTMAX)*10,TEXT*64,SCHM*10,RYEV*10
     &   ,CUBC*10,REIM*10,OCCU*10,OVWR*10,CTHUB*2,CNHUB*1,CLHUB*1
        COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM,NHSORT !!! from init
        COMMON /POTCOM/ SWZ,SMT                              !!! from vfull
        COMMON /PANPAR/ LTPAN,LSPAN,IATS,ISTS                !!! from init
        COMMON /KAPPAR/ NPAN,NKAP                            !!! from init
	COMMON /FSLATR/ FSLU,FSLG                            !!! from readhub
	COMMON /HUBDAT/ NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE,IWR   !!! from readhub
        COMMON /HUBCOM/ DHUB,VHUB,DLDA,VCNS                  !!! from readhub
	COMMON /INP1/   ICHUB                                !!! from readini
	COMMON /INP2/   HUBFILE                              !!! from readini
        COMMON /SPIN/   NSPIN                                !!! from init
        COMMON /EXCH/   IEXCH                                !!! from init
        COMMON /UNIT/   IUN                                  !!! from init
        COMMON /TXTEL/  TXTEL                                !!! from init
        DATA PI/3.141592653589793D0/
	IF(ICHUB.EQ.0)RETURN
C  CHECK EXISTENCE
        INQUIRE(FILE=HUBFILE,EXIST=EXIST)
	IF(.NOT.EXIST)STOP'<READHUB>: HUBFILE MUST EXIST!>'
C  INITIALIZE SLATER INTEGRALS
	DO IATOM=1,NATOM
	DO L1=0,LMAXB
	DO L=0,LMAXB
	DO L2=0,2*LMAXB
	FSLU(L2,L1,L,IATOM)=0.D0
	FSLG(L2,L1,L,IATOM)=0.D0
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C  INITIALIZE OCCUPANCIES "DHUB" AND HUBBARD POTENTIALS "VHUB"
	DO IATOM=1,NATOM
	ISORT=IS(IATOM) 
        DO ISPIN=1,NSPIN                          !!! over spins
	MCRL(IATOM)=0 
        DO L=0,LMAXB                              !!! over l
	LCRL(L,IATOM)=0 
        DO M=-L,L                                 !!! over m
        DO K=-L,L                                 !!! over k
        LM=L*(L+1)+M+1
        LK=L*(L+1)+K+1
        IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
        VHUB(IU,IATOM,ISPIN)=(0.D0,0.D0)
        VCNS(IU,IATOM,ISPIN)=(0.D0,0.D0)
        DHUB(IU,IATOM,ISPIN)=(0.D0,0.D0)
        DLDA(IU,IATOM,ISPIN)=(0.D0,0.D0)
        ENDDO                                     !!! over k
        ENDDO                                     !!! over m
        ENDDO                                     !!! over l
        ENDDO                                     !!! over spins
        ENDDO                                     !!! over atoms
C  READ DATA FROM HUBFILE
	OPEN(UNIT=1,FILE=HUBFILE,STATUS='OLD',FORM='FORMATTED')
	READ(1,1010)TEXT
	READ(1,1010)TEXT
C  **** GENERAL SETTINGS ****
C  SCHM='LDA     ' - simple LDA, (see HUBPOT for details)
C       'LDA+U1.1' - plus Hubbard U, after Liechtestein,
C       'LDA+U1.2' - subtracting LDA occupancies
C       'LDA+U1.3' - subtracting average occupancies 
C       'LDA+U1.4' - spherical case
C       'LDA+C   ' - constrained LDA, potential must be given.
C  RYEV='Ry' or 'eV' - input/output units
C  CUBC='Cubic' or 'Spherical' harmonics
C  REIM='Real' or 'Complex' - input/output
C  OCCU='Y' or 'N' occupation numbers avaiable
C  OVWR='Overwrite' or 'Append' occupation numbers in the HUBFILE
	READ(1,1010)TEXT
	READ(1,*)SCHM
	READ(1,*)RYEV
	READ(1,*)CUBC
	READ(1,*)REIM
	READ(1,*)OCCU
	READ(1,*)OVWR
C  Transform to integer keys
	CALL CONVLOW(SCHM,10)
	CALL RMBLANK(SCHM,10)
	NB=INDEX(SCHM,' ')
	ISCHM=-1
	IF(SCHM(:NB-1).EQ.'lda'      )ISCHM=0
	IF(SCHM(:NB-1).EQ.'lda+u1.1' )ISCHM=11
	IF(SCHM(:NB-1).EQ.'lda+u1.2' )ISCHM=12
	IF(SCHM(:NB-1).EQ.'lda+u1.3' )ISCHM=13
	IF(SCHM(:NB-1).EQ.'lda+u1.4' )ISCHM=14
	IF(SCHM(:NB-1).EQ.'lda+c'    )ISCHM=20
	IF(SCHM(:NB-1).EQ.'lda+cu1.1')ISCHM=21
	IF(SCHM(:NB-1).EQ.'lda+cu1.2')ISCHM=22
	IF(SCHM(:NB-1).EQ.'lda+cu1.3')ISCHM=23
	IF(SCHM(:NB-1).EQ.'lda+cu1.4')ISCHM=24
	IF(ISCHM.EQ.-1)STOP'<READHUB>: SET LDA OR LDA+U# IN HUBFILE'
	CALL CONVLOW(RYEV,10)
	CALL RMBLANK(RYEV,10)
	NB=INDEX(RYEV,' ')
	IF(RYEV(:NB-1).EQ.'ry')RY=1.D0
	IF(RYEV(:NB-1).EQ.'ev')RY=1.D0/13.608D0
	IF(RYEV(:NB-1).NE.'ev'.AND.RYEV(:NB-1).NE.'ry')
     &  STOP'<READHUB>: SET RY OR EV'
	CALL CONVLOW(CUBC,10)
	CALL RMBLANK(CUBC,10)
	NB=INDEX(CUBC,' ')
	ICUBC=0
	IF(CUBC(:NB-1).EQ.'cubic')ICUBC=1
	IF(CUBC(:NB-1).EQ.'spherical')ICUBC=2
	IF(ICUBC.EQ.0)STOP'<READHUB>: SET CUBIC OR SPHERICAL'
	CALL CONVLOW(REIM,10)
	CALL RMBLANK(REIM,10)
	NB=INDEX(REIM,' ')
	IRE=0
	IF(REIM(:NB-1).EQ.'real')IRE=1
	IF(REIM(:NB-1).EQ.'complex')IRE=2
	IF(IRE.EQ.0)STOP'<READHUB>: SET REAL OR COMPLEX'
	CALL CONVLOW(OCCU,10)
	CALL RMBLANK(OCCU,10)
	NB=INDEX(OCCU,' ')
	IOC=0
	IF(OCCU(:NB-1).EQ.'yes')IOC=1
	IF(OCCU(:NB-1).EQ.'no' )IOC=2
	IF(IOC.EQ.0)STOP'<READHUB>: SET YES OR NO TO READ OCC.NUM'
	CALL CONVLOW(OVWR,10)
	CALL RMBLANK(OVWR,10)
	NB=INDEX(OVWR,' ')
	IWR=0
	IF(OVWR(:NB-1).EQ.'overwrite')IWR=1
	IF(OVWR(:NB-1).EQ.'append'   )IWR=2
	IF(IWR.EQ.0)STOP'<READHUB>: SET OVERWRITE OR APPEND'
C  **** CORRELATED STATES *****
C  NHUB  - total number of correlated states
C  IAHUB - atom #
C  CTHUB - <nl> for correlated state, e.g. '3d'
	READ(1,1010)TEXT
	READ(1,*)NHUB                          !!! over correlated states
	DO IHUB=1,NHUB
	READ(1,*)IAHUB,CTHUB
	IF(IAHUB.LE.0.OR.IAHUB.GT.NATOM)STOP'<READHUB>: WRONG ATOM #'
	MCRL(IAHUB)=1
	CNHUB=CTHUB(1:1)
	READ(CNHUB,*)MQ
	CLHUB=CTHUB(2:2)
	CALL CONVLOW(CLHUB,1)
	LHUB=-1
	IF(CLHUB.EQ.'s')LHUB=0
	IF(CLHUB.EQ.'p')LHUB=1
	IF(CLHUB.EQ.'d')LHUB=2
	IF(CLHUB.EQ.'f')LHUB=3
	IF(LHUB.EQ.-1)STOP'<READHUB>: STATES CAN ONLY BE S,P,D,F!'
	IF(LHUB.GT.LMAXB)STOP'<READHUB>: LHUB>LMAXB'
      	LCRL(LHUB,IAHUB)=MQ
	BACKSPACE(1)
C  FHUB - Slater integrals, F(0,ll),F(2,ll),F(4,ll), etc.
	READ(1,*)IAHUB,CTHUB,(FHUB(L2),L2=0,LHUB)
C  SET SLATER INTEGRALS 
	ISORT=IS(IAHUB)
	DO L2=0,2*LHUB,2
	IF(ISCHM.EQ.0)FHUB(L2/2)=0.D0
	FSLU(L2,LHUB,LHUB,IAHUB)=FHUB(L2/2)*RY
	FSLG(L2,LHUB,LHUB,IAHUB)=FHUB(L2/2)*RY
	ENDDO
	ENDDO                                  !!! over correlated states
C  READ PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL IF AVAIABLE
        CALL TRANSS(TS)
C  IDVR=1 corresponds to the occupation numbers for correlated states
C  IDVR=2 corresponds to the occupation numbers from LDA calculation
C  IDVR=3 corresponds to the potential necessary in constrained LDA
C  IDVR=4 corresponds to the potential from LDA+U calculation
 100	READ(1,*,END=200,ERR=200)TEXT
        READ(1,*,END=200,ERR=200)IDVR
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).NE.0)THEN               !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).NE.0)THEN             !!! l_corr
	READ(1,1010)TEXT
	DO ISP=1,2                             !!! over spins
	IF(NSPIN.EQ.1)ISPIN=1
	IF(NSPIN.EQ.2)ISPIN=ISP
C  SET LDA+U OCCUPATION NUMBERS
	DO I=1,IRE
	READ(1,1010)TEXT
	DO M1=-L,L
	READ(1,*)(OCRHUB(M),M=-L,L)
	DO M=-L,L
        IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1  !  (l,m' m) packed
	IF(I.EQ.1)OCPHUB(IU)=DCMPLX(OCRHUB(M),0.D0)
	IF(I.EQ.2)OCPHUB(IU)=DCMPLX(DREAL(OCPHUB(IU)),OCRHUB(M))
	ENDDO
	ENDDO
	ENDDO
C  TRANSFER TO SPHERICAL HARMONICS IF NECESSARY
	IF(ICUBC.EQ.1)THEN
	DO MY1=-L,L
	DO MY =-L,L
        IUY=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MY)+L+MY1+1  !  (l,m' m) packed
	DO MX1=-L,L
	DO MX =-L,L
        IUX=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MX)+L+MX1+1  !  (l,m' m) packed
	IF(IDVR.EQ.1)
     &  DHUB(IUY,IATOM,ISPIN)=DHUB(IUY,IATOM,ISPIN)+
     &  CONJG(TS(MX1+L+1,MY1,L))*OCPHUB(IUX)*TS(MX+L+1,MY,L)
	IF(IDVR.EQ.2)
     &  DLDA(IUY,IATOM,ISPIN)=DLDA(IUY,IATOM,ISPIN)+
     &  CONJG(TS(MX1+L+1,MY1,L))*OCPHUB(IUX)*TS(MX+L+1,MY,L)
	IF(IDVR.EQ.3)
     &  VCNS(IUY,IATOM,ISPIN)=VCNS(IUY,IATOM,ISPIN)+
     &  CONJG(TS(MX1+L+1,MY1,L))*OCPHUB(IUX)*TS(MX+L+1,MY,L)/
     &  2.D0*DBLE(NSPIN)
	IF(IDVR.EQ.4)
     &  VHUB(IUY,IATOM,ISPIN)=VHUB(IUY,IATOM,ISPIN)+
     &  CONJG(TS(MX1+L+1,MY1,L))*OCPHUB(IUX)*TS(MX+L+1,MY,L)/
     &  2.D0*DBLE(NSPIN)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(ICUBC.EQ.2)THEN
	DO M1=-L,L
	DO M=-L,L
        IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
	IF(IDVR.EQ.1)
     &	DHUB(IU,IATOM,ISPIN)=DHUB(IU,IATOM,ISPIN)+OCPHUB(IU)
	IF(IDVR.EQ.2)
     &	DLDA(IU,IATOM,ISPIN)=DLDA(IU,IATOM,ISPIN)+OCPHUB(IU)
	IF(IDVR.EQ.3)
     &	VCNS(IU,IATOM,ISPIN)=VCNS(IU,IATOM,ISPIN)+OCPHUB(IU)/
     &  2.D0*DBLE(NSPIN)
	IF(IDVR.EQ.4)
     &	VHUB(IU,IATOM,ISPIN)=VHUB(IU,IATOM,ISPIN)+OCPHUB(IU)/
     &  2.D0*DBLE(NSPIN)
	ENDDO
	ENDDO
	ENDIF
	ENDDO                                  !!! over spins
	ENDIF                                  !!! l_corr
	ENDDO                                  !!! over l
	ENDIF                                  !!! at_corr
	ENDDO                                  !!! over atoms
	GOTO100
 200	CLOSE(1)
1010	FORMAT(A)
	END



	SUBROUTINE WRITEHUB
*******************************************************
*  Writes occupation numbers to HUBFILE.              * 
*******************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        INCLUDE 'PARAM.DAT'
        INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX)
     &   ,NKAP(NPANMAX),IATS(NPAT,NPANMAX),ISTS(NSORTMAX,NPANMAX)
     &   ,IS(NPAT),NHSORT(NSORTMAX),MCRL(NPAT),LCRL(0:LMAXB,NPAT)
        DIMENSION SMT(NSORTMAX),FHUB(0:LMAXB)
     &   ,FSLG(0:2*LMAXB,0:LMAXB,0:LMAXB,NPAT),OCRHUB(-LMAXB:LMAXB)
     &   ,FSLU(0:2*LMAXB,0:LMAXB,0:LMAXB,NPAT),OCRLDA(-LMAXB:LMAXB)
        COMPLEX*16 TS(7,-3:3,0:3),OCPHUB(LIMLMB2),OCPLDA(LIMLMB2)
     &   ,DHUB(LIMLMB2,NPAT,NSPINMAX),DLDA(LIMLMB2,NPAT,NSPINMAX)
     &   ,VHUB(LIMLMB2,NPAT,NSPINMAX),VCNS(LIMLMB2,NPAT,NSPINMAX)
        LOGICAL LSPAN(0:LMAXT+1,NSORTMAX,NKAPMAX,NPANMAX)
     &   ,LTPAN(0:LMAXT+1,NPAT,NKAPMAX,NPANMAX),EXIST
        CHARACTER*60 HUBFILE
        CHARACTER TXTEL(NSORTMAX)*10,TEXT*64,SCHM*10,RYEV*10
     &   ,CUBC*10,REIM*10,OCCU*10,OVWR*10,CTHUB*2,CNHUB*1,CLHUB*1
     &   ,TL(0:3)*1,TCUBC(-3:3,0:3,2)*10,TPR(-3:3,-3:3)*10,TDP*10
     &   ,TRE(2)*4,TSP(2)*2
        COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM,NHSORT !!! from init
        COMMON /POTCOM/ SWZ,SMT                              !!! from vfull
        COMMON /PANPAR/ LTPAN,LSPAN,IATS,ISTS                !!! from init
        COMMON /KAPPAR/ NPAN,NKAP                            !!! from init
	COMMON /FSLATR/ FSLU,FSLG                            !!! from readhub
	COMMON /HUBDAT/ NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE,IWR   !!! from readhub
        COMMON /HUBCOM/ DHUB,VHUB,DLDA,VCNS                  !!! from readhub
        COMMON /SPIN/   NSPIN                                !!! from init
	COMMON /INP1/   ICHUB                                !!! from readini
	COMMON /INP2/   HUBFILE                              !!! from readini
        COMMON /UNIT/   IUN                                  !!! from init
        COMMON /TXTEL/  TXTEL                                !!! from init
        DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
     &      ,TRE/'REAL','IMAG'/,TSP/'UP','DN'/
	IF(ICHUB.EQ.0)RETURN
C  CHECK EXISTENCE
        INQUIRE(FILE=HUBFILE,EXIST=EXIST)
	IF(.NOT.EXIST)STOP'<READHUB>: HUBFILE MUST EXIST!>'
C  SKIP DATA FROM HUBFILE
	OPEN(UNIT=1,FILE=HUBFILE,STATUS='OLD',FORM='FORMATTED')
	READ(1,1010)TEXT
	READ(1,1010)TEXT
C  **** GENERAL SETTINGS ****
	READ(1,1010)TEXT
	READ(1,*)SCHM
	READ(1,*)RYEV
	READ(1,*)CUBC
	READ(1,*)REIM
	READ(1,*)OCCU
	READ(1,*)OVWR
C  **** CORRELATED STATES *****
	READ(1,1010)TEXT
	READ(1,*)NHUB                          !!! over correlated states
	DO IHUB=1,NHUB
	READ(1,*)IAHUB,CTHUB
	ENDDO                                  !!! over correlated states
C  SKIP PARTIAL OCCUPATION NUMBERS IF NECESSARY
	IF(IWR.EQ.2)THEN                       !!! iwr.eq.2
 100	READ(1,1010,END=200,ERR=200)TEXT
	IF(TEXT(1:9).EQ.' !!!! NEW')BACKSPACE(1)
	IF(TEXT(1:9).EQ.' !!!! NEW')GOTO200 
	GOTO 100
 200	CONTINUE
	ENDIF                                  !!! iwr.eq.2
C  SKIPPING IS DONE, WRITE NEW OCCUPATION NUMBERS
	IF(IWR.EQ.2)WRITE(1,1020)
	CALL PRINTM(1) 
	CALL PRINTH(1) 
	CLOSE(1)
1010	FORMAT(A)
1020	FORMAT(1X,'!!!! NEW OCCUPATION NUMBERS BELOW !!!!')
	END




	SUBROUTINE PRINTM(ICHAN)
*******************************************************
*  Writes occupation numbers to the channel=ICHAN     * 
*******************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        INCLUDE 'PARAM.DAT'
        INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX)
     &   ,NKAP(NPANMAX),IATS(NPAT,NPANMAX),ISTS(NSORTMAX,NPANMAX)
     &   ,IS(NPAT),NHSORT(NSORTMAX),MCRL(NPAT),LCRL(0:LMAXB,NPAT)
        DIMENSION SMT(NSORTMAX),FHUB(0:LMAXB)
     &   ,FSLG(0:2*LMAXB,0:LMAXB,0:LMAXB,NPAT),OCRHUB(-LMAXB:LMAXB)
     &   ,FSLU(0:2*LMAXB,0:LMAXB,0:LMAXB,NPAT),OCRLDA(-LMAXB:LMAXB)
        COMPLEX*16 TC(-3:3,7,0:3),OCPHUB(LIMLMB2),OCPLDA(LIMLMB2)
     &   ,DHUB(LIMLMB2,NPAT,NSPINMAX),DLDA(LIMLMB2,NPAT,NSPINMAX)
     &   ,VHUB(LIMLMB2,NPAT,NSPINMAX),VCNS(LIMLMB2,NPAT,NSPINMAX)
        LOGICAL LSPAN(0:LMAXT+1,NSORTMAX,NKAPMAX,NPANMAX)
     &   ,LTPAN(0:LMAXT+1,NPAT,NKAPMAX,NPANMAX),EXIST
        CHARACTER*60 HUBFILE
        CHARACTER TXTEL(NSORTMAX)*10,TEXT*64,SCHM*10,RYEV*10
     &   ,CUBC*10,REIM*10,OCCU*10,OVWR*10,CTHUB*2,CNHUB*1,CLHUB*1
     &   ,TL(0:3)*1,TCUBC(-3:3,0:3,2)*10,TPR(-3:3,-3:3)*10,TDP*10
     &   ,TRE(2)*4,TSP(2)*2
        COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM,NHSORT !!! from init
        COMMON /POTCOM/ SWZ,SMT                              !!! from vfull
        COMMON /PANPAR/ LTPAN,LSPAN,IATS,ISTS                !!! from init
        COMMON /KAPPAR/ NPAN,NKAP                            !!! from init
	COMMON /FSLATR/ FSLU,FSLG                            !!! from readhub
	COMMON /HUBDAT/ NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE,IWR   !!! from readhub
        COMMON /HUBCOM/ DHUB,VHUB,DLDA,VCNS                  !!! from readhub
        COMMON /SPIN/   NSPIN                                !!! from init
	COMMON /INP1/   ICHUB                                !!! from readini
	COMMON /INP2/   HUBFILE                              !!! from readini
        COMMON /UNIT/   IUN                                  !!! from init
        COMMON /TXTEL/  TXTEL                                !!! from init
        DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
     &      ,TRE/'REAL','IMAG'/,TSP/'UP','DN'/
	IF(ICHUB.EQ.0)RETURN
	CALL INTCUBC(TCUBC)
        CALL TRANSC(TC)
C  SET LDA OCCUPANCIES IF NECESSARY
	IF(ISCHM.EQ.0)THEN
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	DO ISPIN=1,NSPIN
        DO L=0,ILSB(ISORT)
        DO M1=-L,L
        LM1=L*(L+1)+M1+1
        DO M=-L,L
        LM=L*(L+1)+M+1
        IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1 ! ! (l,m'm)
        DLDA(IU,IATOM,ISPIN)=DHUB(IU,IATOM,ISPIN)
        ENDDO
        ENDDO
        ENDDO
	ENDDO
	ENDDO                                  !!! over atoms
	ENDIF
C  PRINT OCCUPANCIES
	DO 1000 IDVR=1,2                       !!! over LDA+U,LDA
	WRITE(ICHAN,1010)
	IF(IDVR.EQ.1)WRITE(ICHAN,1021)
	IF(IDVR.EQ.2)WRITE(ICHAN,1022)
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
        DO I=10,1,-1
        IF(TXTEL(ISORT)(I:I).NE.' ')EXIT
        ENDDO
        NB=I
	IF(MCRL(IATOM).NE.0)THEN               !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).NE.0)THEN             !!! l_corr
	WRITE(ICHAN,1030)LCRL(L,IATOM),TL(L),TXTEL(ISORT)(:NB)
	DO ISP=1,2                             !!! over spins
	IF(NSPIN.EQ.1)ISPIN=1
	IF(NSPIN.EQ.2)ISPIN=ISP
C  TRANSFORM TO CUBIC HARMONICS IF REQUIRED
	IF(ICUBC.EQ.1)THEN
	DO MX1=-L,L
	DO MX =-L,L
        IUX=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MX)+L+MX1+1  !  (l,m' m) packed
	OCPHUB(IUX)=(0.D0,0.D0)
	DO MY1=-L,L
	DO MY =-L,L
        IUY=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MY)+L+MY1+1  !  (l,m' m) packed
	IF(IDVR.EQ.1)
     &	OCPHUB(IUX)=OCPHUB(IUX)+
     &  CONJG(TC(MY1,MX1+L+1,L))*DHUB(IUY,IATOM,ISPIN)*TC(MY,MX+L+1,L)
	IF(IDVR.EQ.2)
     &	OCPHUB(IUX)=OCPHUB(IUX)+
     &  CONJG(TC(MY1,MX1+L+1,L))*DLDA(IUY,IATOM,ISPIN)*TC(MY,MX+L+1,L)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(ICUBC.EQ.2)THEN
	DO M1=-L,L
	DO M=-L,L
        IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
	IF(IDVR.EQ.1)
     &	OCPHUB(IU)=DHUB(IU,IATOM,ISPIN)
	IF(IDVR.EQ.2)
     &	OCPHUB(IU)=DLDA(IU,IATOM,ISPIN)
	ENDDO
	ENDDO
	ENDIF
C  WRITE REAL/IMAG PARTS SEPARATELY
	DO I=1,IRE                             !!! over re/im
	IF(L.EQ.0)
     &  WRITE(ICHAN,1040)(TCUBC(M,L,ICUBC),M=-L,L),
     &  TRE(I),TSP(ISP)
	IF(L.EQ.1)
     &  WRITE(ICHAN,1041)(TCUBC(M,L,ICUBC),M=-L,L),
     &  TRE(I),TSP(ISP)
	IF(L.EQ.2)
     &  WRITE(ICHAN,1042)(TCUBC(M,L,ICUBC),M=-L,L),
     &  TRE(I),TSP(ISP)
	IF(L.EQ.3)
     &  WRITE(ICHAN,1043)(TCUBC(M,L,ICUBC),M=-L,L),
     &  TRE(I),TSP(ISP)
	DO M1=-L,L
	DO M=-L,L
        IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
        IF(I.EQ.1)TPAR=DREAL(OCPHUB(IU))/2.D0*DBLE(NSPIN)
        IF(I.EQ.2)TPAR=DIMAG(OCPHUB(IU))/2.D0*DBLE(NSPIN)
        IF(ABS(TPAR).GT.1.D-07)TPR(M1,M)=TDP(TPAR,1.D-07)
        IF(ABS(TPAR).LE.1.D-07)TPR(M1,M)=' 0.0000000'
	ENDDO
	ENDDO
	DO M1=-L,L
	IF(L.EQ.0)WRITE(ICHAN,1050)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.1)WRITE(ICHAN,1051)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.2)WRITE(ICHAN,1052)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.3)WRITE(ICHAN,1053)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	ENDDO
	ENDDO                                  !!! over re/im
	ENDDO                                  !!! over spins
	ENDIF                                  !!! l_corr
	ENDDO                                  !!! over l
	ENDIF                                  !!! at_corr
	ENDDO                                  !!! over atoms
1000	CONTINUE                               !!! over LDA,U occupancies 
1010	FORMAT(64('='))
1021	FORMAT(1X,'1  -  TABLE OF OCCUPATION NUMBERS FOR LDA+U :')
1022	FORMAT(1X,'2  -  TABLE OF OCCUPATION NUMBERS FOR LDA   :')
1030	FORMAT(1X,'State ',I1,A1,1X,A,
     &  '; spin up/dn occupation numbers are :')
1040	FORMAT(1X,1(A10,2X),2X,A4,',',A2,',',A5)
1041	FORMAT(1X,3(A10,2X),2X,A4,',',A2,',',A5)
1042	FORMAT(1X,5(A10,2X),2X,A4,',',A2,',',A5)
1043	FORMAT(1X,7(A10,2X),2X,A4,',',A2,',',A5)
1050    FORMAT(1X,1(A10,2X),2X,A10)
1051    FORMAT(1X,3(A10,2X),2X,A10)
1052    FORMAT(1X,5(A10,2X),2X,A10)
1053    FORMAT(1X,7(A10,2X),2X,A10)
	END





	SUBROUTINE PRINTH(ICHAN)
*******************************************************
*  Prints Hubbard correction to the LDA potential     * 
*******************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        INCLUDE 'PARAM.DAT'
        INTEGER ILST(NSORTMAX),ILSB(NSORTMAX),ILSV(NSORTMAX)
     &   ,NKAP(NPANMAX),IATS(NPAT,NPANMAX),ISTS(NSORTMAX,NPANMAX)
     &   ,IS(NPAT),NHSORT(NSORTMAX),MCRL(NPAT),LCRL(0:LMAXB,NPAT)
        DIMENSION SMT(NSORTMAX),FHUB(0:LMAXB)
     &   ,FSLG(0:2*LMAXB,0:LMAXB,0:LMAXB,NPAT),OCRHUB(-LMAXB:LMAXB)
     &   ,FSLU(0:2*LMAXB,0:LMAXB,0:LMAXB,NPAT),OCRLDA(-LMAXB:LMAXB)
        LOGICAL LSPAN(0:LMAXT+1,NSORTMAX,NKAPMAX,NPANMAX)
     &   ,LTPAN(0:LMAXT+1,NPAT,NKAPMAX,NPANMAX),EXIST
        COMPLEX*16 TC(-3:3,7,0:3),OCPHUB(LIMLMB2),OCPLDA(LIMLMB2)
     &   ,DHUB(LIMLMB2,NPAT,NSPINMAX),DLDA(LIMLMB2,NPAT,NSPINMAX)
     &   ,VHUB(LIMLMB2,NPAT,NSPINMAX),VCNS(LIMLMB2,NPAT,NSPINMAX)
        CHARACTER*60 HUBFILE
        CHARACTER TXTEL(NSORTMAX)*10,TEXT*64,SCHM*10,RYEV*10
     &   ,CUBC*10,REIM*10,OCCU*10,OVWR*10,CTHUB*2,CNHUB*1,CLHUB*1
     &   ,TL(0:3)*1,TCUBC(-3:3,0:3,2)*10,TPR(-3:3,-3:3)*10,TDP*10
     &   ,TRE(2)*4,TSP(2)*2
        COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM,NHSORT !!! from init
        COMMON /POTCOM/ SWZ,SMT                              !!! from vfull
        COMMON /PANPAR/ LTPAN,LSPAN,IATS,ISTS                !!! from init
        COMMON /KAPPAR/ NPAN,NKAP                            !!! from init
	COMMON /FSLATR/ FSLU,FSLG                            !!! from readhub
	COMMON /HUBDAT/ NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE,IWR   !!! from readhub
        COMMON /HUBCOM/ DHUB,VHUB,DLDA,VCNS                  !!! from readhub
        COMMON /SPIN/   NSPIN                                !!! from init
	COMMON /INP1/   ICHUB                                !!! from readini
	COMMON /INP2/   HUBFILE                              !!! from readini
        COMMON /UNIT/   IUN                                  !!! from init
        COMMON /TXTEL/  TXTEL                                !!! from init
        DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
     &      ,TRE/'REAL','IMAG'/,TSP/'UP','DN'/
	IF(ICHUB.EQ.0)RETURN
	CALL INTCUBC(TCUBC)
        CALL TRANSC(TC)
C  WRITE HUBBARD CORRECTION TO THE LDA POTENTIAL
	DO 1000 IDVR=3,4                       !!! over idvr
	WRITE(ICHAN,1010)
	IF(IDVR.EQ.3)WRITE(ICHAN,1023)
	IF(IDVR.EQ.4)WRITE(ICHAN,1024)
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
        DO I=10,1,-1
        IF(TXTEL(ISORT)(I:I).NE.' ')EXIT
        ENDDO
        NB=I
	IF(MCRL(IATOM).NE.0)THEN               !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).NE.0)THEN             !!! l_corr
	WRITE(ICHAN,1030)LCRL(L,IATOM),TL(L),TXTEL(ISORT)(:NB)
	DO ISP=1,2                             !!! over spins
	IF(NSPIN.EQ.1)ISPIN=1
	IF(NSPIN.EQ.2)ISPIN=ISP
C  TRANSFORM TO CUBIC HARMONICS IF REQUIRED
	IF(ICUBC.EQ.1)THEN
	DO MX1=-L,L
	DO MX =-L,L
        IUX=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MX)+L+MX1+1  !  (l,m' m) packed
	OCPHUB(IUX)=(0.D0,0.D0)
	DO MY1=-L,L
	DO MY =-L,L
        IUY=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MY)+L+MY1+1  !  (l,m' m) packed
	IF(IDVR.EQ.3)
     &	OCPHUB(IUX)=OCPHUB(IUX)+
     &  CONJG(TC(MY1,MX1+L+1,L))*VCNS(IUY,IATOM,ISPIN)*TC(MY,MX+L+1,L)
	IF(IDVR.EQ.4)
     &	OCPHUB(IUX)=OCPHUB(IUX)+
     &  CONJG(TC(MY1,MX1+L+1,L))*VHUB(IUY,IATOM,ISPIN)*TC(MY,MX+L+1,L)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(ICUBC.EQ.2)THEN
	DO M1=-L,L
	DO M=-L,L
        IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
	IF(IDVR.EQ.3)
     &	OCPHUB(IU)=VCNS(IU,IATOM,ISPIN)
	IF(IDVR.EQ.4)
     &	OCPHUB(IU)=VHUB(IU,IATOM,ISPIN)
	ENDDO
	ENDDO
	ENDIF
C  WRITE REAL/IMAG PARTS SEPARATELY
	DO I=1,IRE                             !!! over re/im
	IF(L.EQ.0)
     &  WRITE(ICHAN,1040)(TCUBC(M,L,ICUBC),M=-L,L),TRE(I),TSP(ISP)
	IF(L.EQ.1)
     &  WRITE(ICHAN,1041)(TCUBC(M,L,ICUBC),M=-L,L),TRE(I),TSP(ISP)
	IF(L.EQ.2)
     &  WRITE(ICHAN,1042)(TCUBC(M,L,ICUBC),M=-L,L),TRE(I),TSP(ISP)
	IF(L.EQ.3)
     &  WRITE(ICHAN,1043)(TCUBC(M,L,ICUBC),M=-L,L),TRE(I),TSP(ISP)
	DO M1=-L,L
	DO M=-L,L
        IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
        IF(I.EQ.1)TPAR=DREAL(OCPHUB(IU))
        IF(I.EQ.2)TPAR=DIMAG(OCPHUB(IU))
        IF(ABS(TPAR).GT.1.D-07)TPR(M1,M)=TDP(TPAR,1.D-07)
        IF(ABS(TPAR).LE.1.D-07)TPR(M1,M)=' 0.0000000'
	ENDDO
	ENDDO
	DO M1=-L,L
	IF(L.EQ.0)WRITE(ICHAN,1050)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.1)WRITE(ICHAN,1051)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.2)WRITE(ICHAN,1052)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.3)WRITE(ICHAN,1053)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	ENDDO
	ENDDO                                  !!! over re/im
	ENDDO                                  !!! over spins
	ENDIF                                  !!! l_corr
	ENDDO                                  !!! over l
	ENDIF                                  !!! at_corr
	ENDDO                                  !!! over atoms
1000	CONTINUE                               !!! over idvr
1010	FORMAT(64('='))
1023	FORMAT(1X,'3  -  CONSTRAINED PART   OF THE LDA POTENTIAL :')
1024	FORMAT(1X,'4  -  HUBBARD CORRECTION TO THE LDA POTENTIAL :')
1030	FORMAT(1X,'State ',I1,A1,1X,A,
     &  '; spin up/dn Hubbard potentials are :')
1040	FORMAT(1X,1(A10,2X),2X,A4,',',A2)
1041	FORMAT(1X,3(A10,2X),2X,A4,',',A2)
1042	FORMAT(1X,5(A10,2X),2X,A4,',',A2)
1043	FORMAT(1X,7(A10,2X),2X,A4,',',A2)
1050    FORMAT(1X,1(A10,2X),2X,A10)
1051    FORMAT(1X,3(A10,2X),2X,A10)
1052    FORMAT(1X,5(A10,2X),2X,A10)
1053    FORMAT(1X,7(A10,2X),2X,A10)
	END
